#include "StdAfx.h"
#include "FlvDataCollector.h"

//=====================================================================================//
//                           struct FlvDataCollector::Buffer                           //
//=====================================================================================//
FlvDataCollector::Buffer::Buffer(const char* data,unsigned int size,unsigned int timestamp)
: m_data(size)
, m_timestamp(timestamp)
{
	memcpy(&m_data[0],data,size);
}

//=====================================================================================//
//                               class FlvDataCollector                                //
//=====================================================================================//
FlvDataCollector::FlvDataCollector(HFTFLVSTREAM hFlvStream,FlvDataCollectorObserver* observer,
								   unsigned int image_width,unsigned int image_height,
								   unsigned int frame_rate)
: m_hScreenVideoEncoder(ftScreenVideoEncoderCreate(image_width,image_height,16)) 
, m_frameDuration(1000/frame_rate)
, m_hFlvStream(hFlvStream)
, m_observer(observer)
, m_videoTimestamp(0)
, m_audioTimestamp(0)
{
}

FlvDataCollector::~FlvDataCollector()
{
	ftScreenVideoEncoderDestroy(m_hScreenVideoEncoder);
}
//	flush data
void FlvDataCollector::flush()
{
	flushStoredBuffers(m_audio,&ftFlvStreamPutAudioData);	
}
//	process video data callback
void FlvDataCollector::onVideoData(const char* data,unsigned int size,unsigned int timestamp)
{
	assert( timestamp >= m_videoTimestamp );

	flushStoredBuffers(m_audio,&ftFlvStreamPutAudioData,timestamp);	

	LPCVOID lpData = 0;
	DWORD cbBytes = 0;

	ftScreenVideoEncodeFrame(m_hScreenVideoEncoder,data,size,true);
	ftScreenVideoGetEncodedFrame(m_hScreenVideoEncoder,&lpData,&cbBytes);
	ftFlvStreamPutVideoData(m_hFlvStream,lpData,cbBytes,timestamp);

	if(m_observer)
		m_observer->onProgress(timestamp);

	m_videoTimestamp = timestamp;
}
//	process audio data callback
void FlvDataCollector::onAudioData(const char* data,unsigned int size,unsigned int timestamp)
{
	assert( timestamp >= m_audioTimestamp );

	if(m_audioTimestamp <= (m_videoTimestamp + m_frameDuration))
	{
		ftFlvStreamPutAudioData(m_hFlvStream,data,size,timestamp);

		if(m_observer)
			m_observer->onProgress(timestamp);
	}
	else
	{
		m_audio.push_back(Buffer(data,size,timestamp));
	}

	m_audioTimestamp = timestamp;
}
//	flush stored data buffers
void FlvDataCollector::flushStoredBuffers(buffers_t& buffers,ftFlvStreamPutData func,unsigned int timestamp)
{
	if(!buffers.empty())
	{
		unsigned int counter = 0;

		for(unsigned int i=0;i<buffers.size();++i)
		{
			const Buffer& buffer = buffers[i];
			if(buffer.m_timestamp < timestamp)
			{
				func(m_hFlvStream,&buffer.m_data[0],buffer.m_data.size(),buffer.m_timestamp);
				++counter;
			}
			else break;
		}

		if(counter)
			buffers.erase(buffers.begin(),buffers.begin()+counter);
	}
}
//	flush all stored data buffers
void FlvDataCollector::flushStoredBuffers(buffers_t& buffers,ftFlvStreamPutData func)
{
	if(!buffers.empty())
	{
		for(unsigned int i=0;i<buffers.size();++i)
		{
			const Buffer& buffer = buffers[i];
			func(m_hFlvStream,&buffer.m_data[0],buffer.m_data.size(),buffer.m_timestamp);
		}

		buffers.clear();
	}
}